# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# manage_blocage.py
#
# Librairie de gestion du blocage réseau des stations
#
###########################################################################

from twisted.internet import reactor
from twisted.python import log
import sys
sys.path.append('/usr/share/eole/controlevnc')

from connexions import Connexions
from cliscribe import Cliscribe
import ldap_utils

class GestBlocage:
    """Gestion des blocages réseau
    """
    def __init__(self):
        #self.bloc_dict = dict()
        self.bloc_dict = {}
        self.bd_list = { 'reseau' : { 0 : 'Aucun', 1 : 'Tout internet'},
                         'partages' : { 0 : 'Normaux', 1 : u'Aucun lecteur', 2 : 'Seulement le partage "devoirs"'}}
        self.mod_dict = {'netmod' : 0 , 'partmod' : (0, None)}

    def get_bloc_list(self):
        """renvoie la liste des types de blocage
        """
        return self.bd_list
        
    def get_net_bloc(self, user):
        """blocage reseau pour user
        """
        return self.get_blocage(user)['netmod']

    def get_part_bloc(self, user):
        """type partage pour user
        """
        return self.get_blocage(user)['partmod']

    def get_blocage(self, user):
        """type de blocage pour <user> ou type par défaut
        """
        if self.bloc_dict.has_key(user): return self.bloc_dict[user]
        return self.mod_dict
  
    def set_blocage(self, user, netmod, partmod, duree):
        """active le blocage netmod pour user pendant duree
        renseigne le dictionnaire { user : type_blocage }
        """
        netmode, duree = int(netmod), int(duree)*60
        if netmod == 0 and partmod[0] == 0:
            self.purge_l(user)
        else:
            self.bloc_dict[user] = {'netmod' : netmod , 'partmod' : partmod}
            reactor.callLater(duree, self.purge_l, user)
            self.bloc_user(user)
        
    def purge_l(self, user):
        """supprime user de self.bloc_dict
        """
        try :del(self.bloc_dict[user])
        except: pass
        self.bloc_user(user)

    def bloc_user(self, user, ip=None, logon=False):
        """active le blocage sur le poste où est connecté user (s'il est connecté bien sûr)
        """
#        log.msg('bloc_user %s %s %s'%(user, ip, logon))
        if not Connexions().is_connecte(user): return #rien a faire si non connecte
        if not ip:
            try: ip = Connexions().get_ip(user)
            except: return True
        if type(ip) == str: 
            d = self.bloc_partage(user=user, ip=ip, logon=logon)
            d.addCallback(self.bloc_net, user=user, ip=ip)
            return d
        elif type(ip) == list and ip:
            d = self.bloc_partage(user=user, ip=ip[0], logon=logon)
            d.addCallback(self.bloc_net, user=user, ip=ip[0])
            for i in ip[1:]:
                d.addCallback(self.bloc_partage, user=user, ip=i, logon=logon)
                d.addCallback(self.bloc_net, user=user, ip=i)
            return d

    def bloc_partage(self, ret=None, user=None, ip=None, logon=False):
#        log.msg('bloc partage %s'%ip)
        sid = ldap_utils.get_user_sid(user)
        partmod = self.get_part_bloc(user)
        return Cliscribe(ip).bloc(partmod=partmod, sid=sid, logon=logon)

    def bloc_net(self, ret=None, user=None, ip=None):
#        log.msg('bloc net %s ip=%s'%(ret, ip))
        netmod = self.get_net_bloc(user)
        if netmod == 1:
            return Cliscribe(ip).fw(cmd='SETMODE::block;;block')
        elif netmod == 0:
            return self.init_net(ip)

    def init_net(self, ip):
        return Cliscribe(ip).fw(cmd='SETMODE::block;;allow')

